
#include "cobject.h"
#include <qmath.h>
#include <iostream>

using namespace std;

CObject::CObject()
    : m_count(0)
{

    // allocates memory for object data
    m_data.resize(36 * 6);

    // puts cube on the scene
    cube(1.0f, 1.0f, 1.0f);

    cout << "Number of allocated GLfloats: " << m_count << endl;
}


// Adds object data to data buffer (vertex position and vertex normal vector), m_count is set to number of allocated GLfloat's
void CObject::add(const QVector3D &v, const QVector3D &n)
{
    GLfloat *p = m_data.data() + m_count;
    *p++ = v.x();
    *p++ = v.y();
    *p++ = v.z();
    *p++ = n.x();
    *p++ = n.y();
    *p++ = n.z();
    m_count += 6;
}

// Creates the quad object consisting of two triangles and normal vector
void CObject::quad3(GLfloat x1, GLfloat y1, GLfloat z1,
                 GLfloat x2, GLfloat y2, GLfloat z2,
                 GLfloat x3, GLfloat y3, GLfloat z3,
                 GLfloat x4, GLfloat y4, GLfloat z4)
{
    QVector3D n = QVector3D::normal(QVector3D(x4 - x1, y4 - y1, z4 - z1), QVector3D(x2 - x1, y2 - y1, z2 - z1));

    add(QVector3D(x1, y1, z1), n);
    add(QVector3D(x4, y4, z4), n);
    add(QVector3D(x2, y2, z2), n);

    add(QVector3D(x3, y3, z3), n);
    add(QVector3D(x2, y2, z2), n);
    add(QVector3D(x4, y4, z4), n);
}

// Creates the cube located at (0,0,0) with width, height, and depth equal to (w,h,d) respectively.
void CObject::cube(GLfloat w, GLfloat h, GLfloat d)
{
    quad3(-w/2, -h/2, -d/2,  +w/2, -h/2, -d/2,  +w/2, +h/2, -d/2,  -w/2, +h/2, -d/2);
    quad3(-w/2, -h/2, +d/2,  -w/2, +h/2, +d/2,  +w/2, +h/2, +d/2,  +w/2, -h/2, +d/2);

    quad3(-w/2, -h/2, -d/2,  -w/2, -h/2, +d/2,  +w/2, -h/2, +d/2,  +w/2, -h/2, -d/2);
    quad3(-w/2, +h/2, -d/2,  +w/2, +h/2, -d/2,  +w/2, +h/2, +d/2,  -w/2, +h/2, +d/2);

    quad3(+w/2, -h/2, -d/2,  +w/2, -h/2, +d/2,  +w/2, +h/2, +d/2,  +w/2, +h/2, -d/2);
    quad3(-w/2, -h/2, -d/2,  -w/2, +h/2, -d/2,  -w/2, +h/2, +d/2,  -w/2, -h/2, +d/2);
}








